@@ -4,7 +4,7 @@
  **/ 
 
 #include "cgiwrap.h"	/* Headers for all CGIwrap source files */
-RCSID("$Id: cgiwrap.c,v 1.37 2001/07/22 17:22:07 nneul Exp $");
+RCSID("$Id: cgiwrap.c,v 1.47 2003/10/16 16:40:57 nneul Exp $");
 
 /*
  * Global context structure
@@ -21,6 +21,7 @@ int main (int argc, char *argv[])
 	char *scriptPath; /* Path to script file */
 	char *cgiBaseDir; /* Base directory for cgi scripts in user's dir */
 	struct passwd *user; /* For getting uid from name */
+	char *interPath = NULL; /* Path to interpreter - i.e. php */
 #ifdef CONF_REPORT_RUSAGE
 	pid_t childpid;
 	struct rusage childrusage;
@@ -33,6 +34,12 @@ int main (int argc, char *argv[])
 	
 	/* Initially not a multiuser cgi script */
 	Context.multiuser_cgi_script = 0;
+	Context.interpreted_script = 0;
+
+#ifdef CONF_QUIET_ERRORS
+	/* Default to quiet errors only */
+	MSG_QuietErrors = 1;
+#endif
 
 	/* Determine if debugging output should be done */
 	if ( strlen(argv[0]) >= 1 )
@@ -40,6 +47,7 @@ int main (int argc, char *argv[])
 		CONF_DEBUG = !strcmp( argv[0] + strlen(argv[0]) - 1, "d");
 		if (CONF_DEBUG)
 		{
+			MSG_QuietErrors = 0;
 			MSG_HTMLMessages = 0;
 		}
 	}
@@ -49,6 +57,13 @@ int main (int argc, char *argv[])
 	{
 		MSG_Need_NPH_Header = 1;
 	}
+#if defined(PATH_PROG_PHP)
+	else if ( !strncmp(argv[0], "php-", 4) )
+	{
+		Context.interpreted_script = 1;
+		interPath = PATH_PROG_PHP;
+	}
+#endif
 	
 	/* Output a Content-type header if in debugging mode */
 	if ( CONF_DEBUG )
@@ -82,6 +97,9 @@ int main (int argc, char *argv[])
 
 	/* Output the contents of important environment variables */
 	OutputEnvironment();
+
+	/* Save copies of the original values for use later */
+	SaveEnvironment();
 	
 	/* Get the user name from the given data */
 	userStr = FetchUserString();
@@ -102,7 +120,9 @@ int main (int argc, char *argv[])
 	DEBUG_Str("   Home Dir:", user->pw_dir);
 
 	/* Perform checks to make sure this user is allow to use CGI scripts */
-	CheckUser(user);
+	CheckUser(user); /* minuid/mingid/shell */
+	CheckUserAccess(user); /* global config files */
+	CheckVHostUserAccess(user); /* vhost config files */
 
 	/* Determine the base directory where this user's CGI scripts
 		are to be stored */
@@ -121,9 +141,9 @@ int main (int argc, char *argv[])
 	}
 
 	/* Get the script name from the given data */
-DEBUG_Msg("\tFetching script string\n");
+	DEBUG_Msg("\tFetching script string\n");
 	scrStr = FetchScriptString(cgiBaseDir);
-DEBUG_Msg("\tBuilding script path\n");
+	DEBUG_Msg("\tBuilding script path\n");
 #ifdef CONF_MULTIUSER_CGI_DIR
 	if ( Context.multiuser_cgi_script )
 	{
@@ -134,13 +154,49 @@ DEBUG_Msg("\tBuilding script path\n");
 	{
 		scriptPath = BuildScriptPath(cgiBaseDir,scrStr);
 	}
-DEBUG_Msg("\tCondensing slashes.\n");
+
+	DEBUG_Msg("\tCondensing slashes.\n");
 	Context.scriptFullPath = CondenseSlashes(scriptPath);
 	Context.scriptRelativePath = CondenseSlashes(scrStr);
 
 	DEBUG_Str("\tScript Relative Path: ", scrStr);
 	DEBUG_Str("\tScript Absolute Path: ", scriptPath);
 
+#if defined(CONF_PHP_INTERPRETER) && defined(PATH_PROG_PHP)
+	DEBUG_Msg("\tChecking for special interpreted script (php).");
+	/* don't double check if already php-cgiwrap */
+	if ( ! interPath )
+	{
+
+		if ( 
+			StringEndsWith(scriptPath, ".php") ||
+			StringEndsWith(scriptPath, ".php3") ||
+			StringEndsWith(scriptPath, ".php4") ||
+			StringEndsWith(scriptPath, ".phtml") )
+		{
+			Context.interpreted_script = 1;
+			interPath = PATH_PROG_PHP;
+		}
+	}
+#endif
+#if defined(CONF_ASP_INTERPRETER) && defined(PATH_PROG_ASP)
+	DEBUG_Msg("\tChecking for special interpreted script (asp).");
+	if ( ! interPath )
+	{
+
+		if ( StringEndsWith(scriptPath, ".asp") )
+		{
+			Context.interpreted_script = 1;
+			interPath = PATH_PROG_ASP;
+		}
+	}
+#endif
+
+	if ( interPath )
+	{
+		DEBUG_Str("\tInterpreter Path: ", interPath);
+	}
+
 	/* Set the Correct Values of environment variables */
 	DEBUG_Msg("\nFixing Environment Variables.");
 	SetScriptName(userStr, scrStr);
@@ -189,6 +245,15 @@ DEBUG_Msg("\tCondensing slashes.\n");
 	/* Check to see if ok to execute script file */
 	CheckScriptFile();
 
+	/* Check to see if we should force handling with #! line */
+#if defined(CONF_PHP_NONEXEC_ONLY)
+	if ( Context.interpreted_script && Context.script_is_executable )
+	{
+		DEBUG_Msg("\nScript is executable, ignoring interpreter and using #! line.");
+		Context.interpreted_script = 0;
+	}
+#endif
+
 	/* Perform any AFS related tasks before executing script */
 	Create_AFS_PAG();
 	
@@ -207,7 +272,14 @@ DEBUG_Msg("\tCondensing slashes.\n");
 	}
 	else if ( childpid == 0 )
 	{
-		execv(scriptPath, CreateARGV(scrStr, argc,argv));
+		if ( interPath && Context.interpreted_script )
+		{
+			execv(interPath, CreateInterpreterARGV(interPath, scrStr, argc, argv));
+		}
+		else
+		{
+			execv(scriptPath, CreateARGV(scrStr, argc, argv));
+		}
 		MSG_Error_ExecFailed();
 		Log(userStr, scrStr, "failed execv of script");
 		exit(1);
@@ -229,7 +301,14 @@ DEBUG_Msg("\tCondensing slashes.\n");
 		exit(WEXITSTATUS(childstatus));
 	}
 #else
-	execv(scriptPath, CreateARGV(scrStr, argc,argv));
+	if ( interPath && Context.interpreted_script )
+	{
+		execv(interPath, CreateInterpreterARGV(interPath, scrStr, argc, argv));
+	}
+	else
+	{
+		execv(scriptPath, CreateARGV(scrStr, argc, argv));
+	}
 	MSG_Error_ExecFailed();
 	exit(1);
 #endif
