@@ -1,4 +1,4 @@
-/* rcsid: $Id: msgs.h,v 1.9 2001/07/22 17:22:07 nneul Exp $ */
+/* rcsid: $Id: msgs.h,v 1.14 2003/09/26 22:19:08 nneul Exp $ */
 
 /**
  **  File: messages.h
@@ -6,6 +6,7 @@
  **/ 
 
 extern int MSG_HTMLMessages;
+extern int MSG_QuietErrors;
 extern int MSG_Need_NPH_Header;
 
 void MSG_ContentType(char *typestring);
@@ -19,10 +20,13 @@ void MSG_Error_General(char *message);
 void MSG_Error_ServerUser(char *msg);
 void MSG_Error_ExecutionNotPermitted(char *path, char *reason);
 void MSG_Error_SystemError(char *when);
-void MSG_Error_AccessControl(char *why, char *controlfile);
+void MSG_Error_AccessControl(char *why, char *allowfile, char *denyfile);
 void MSG_Error_ExecFailed(void);
 void MSG_Error_NoSuchUser(char *user);
 void MSG_Error_NoScriptDir(void);
+void MSG_Error_ServerConfigError(void);
+void MSG_Error_RequestError(void);
+
 void MSG_Info(void);
 void MSG_Error_ServerUserMismatch(void);
 void MSG_Error_ServerUserNotFound(void);
