@@ -4,7 +4,7 @@
  **/ 
 
 #include "cgiwrap.h"	/* Headers for all CGIwrap source files */
-RCSID("$Id: util.c,v 1.50 2001/10/30 15:10:15 nneul Exp $");
+RCSID("$Id: util.c,v 1.72 2003/10/24 14:54:09 nneul Exp $");
 
 /*
  * Encode string to protect against cross-site scripting, but only
@@ -142,6 +142,29 @@ char **CreateARGV( char *scrStr, int argc, char *argv[])
 }
 
 
+/*
+ * Build the ARGV array for passing to the called script via interpreter
+ */
+char **CreateInterpreterARGV( char *interp, char *scrStr, int argc, char *argv[])
+{
+	char **temp;
+	int i;
+
+	temp = (char **) SafeMalloc( (argc+2) * sizeof(char *), "ARGV array");
+	
+	temp[0] = StripPathComponents( CountSubDirs(interp), interp );
+	temp[1] = StripPathComponents( CountSubDirs(scrStr), scrStr );
+	temp[argc+1] = 0;
+
+	for (i=1; i<argc; i++)
+	{
+		temp[i+1] = argv[i];
+	}
+
+	return temp;
+}
+
+
 /*
  *   Extract and return the value portion of a key=value pair found in a string
  */
@@ -179,6 +202,23 @@ char *GetValue( char *keyword, char *string )
 	return returnStr;
 }
 
+/*
+ * Check if a script name ends with a particular string (i.e. .php)
+ */
+int StringEndsWith(char *string, char *ending)
+{
+	int len = strlen(string);
+	int elen = strlen(ending);
+
+	if ( len >= elen && !strncmp(string + len - elen, ending, elen) )
+	{
+		return 1;
+	}
+	else
+	{
+		return 0;
+	}
+}
 
 /*
  * Check if a path is safe to use
@@ -258,6 +298,7 @@ void OutputEnvironment(void)
 	DEBUG_Str("     QUERY_STRING:", (char *) getenv("QUERY_STRING") );
         DEBUG_Str("      SCRIPT_NAME:", (char *) getenv("SCRIPT_NAME") );
         DEBUG_Str("  SCRIPT_FILENAME:", (char *) getenv("SCRIPT_FILENAME") );
+        DEBUG_Str("     REDIRECT_URL:", (char *) getenv("REDIRECT_URL") );
         DEBUG_Str("        PATH_INFO:", (char *) getenv("PATH_INFO") );
         DEBUG_Str("  PATH_TRANSLATED:", (char *) getenv("PATH_TRANSLATED") );
         DEBUG_Str("      REMOTE_USER:", (char *) getenv("REMOTE_USER") );
@@ -265,6 +306,20 @@ void OutputEnvironment(void)
         DEBUG_Str("      REMOTE_ADDR:", (char *) getenv("REMOTE_ADDR") );
 }
 
+/* 
+ * Save a copy of the old environment variables 
+ */
+void SaveEnvironment(void)
+{
+	char *c;
+
+	c = getenv("PATH_INFO");
+	if ( c )
+	{
+		Context.origPathInfo = strdup(c);
+	}
+}
+
 /*
  * Change to the users cgi-bin directory, this serves to provide
  * a consistent strting point for paths.
@@ -295,83 +350,37 @@ void ChangeToCGIDir(char *scriptPath)
 }
 
 /*
- * Perform checks on the userid 
+ * Perform checks on the userid (global checks)
  */
 void CheckUser(struct passwd *user)
 {
-#if defined(CONF_DENYFILE) || defined(CONF_ALLOWFILE)
-	int deny_exists=0, allow_exists=0;
-	int in_deny, in_allow;	
-
 #if defined(CONF_CHECKHOST)
+	DEBUG_Msg("Checking remote host information.");
 	if ( ( !getenv("REMOTE_ADDR") ) || (getenv("REMOTE_ADDR")[0] == '\0') )
 	{
 		Log(user->pw_name, "-", "no remote host");
-		MSG_Error_General("Your host (null) is not allowed to run this");
+		MSG_Error_General("Cannot determine if your host is allowed to run this script.");
 	}
 #endif
 
-#if defined(CONF_DENYFILE)
-	deny_exists = FileExists(CONF_DENYFILE);
-#endif
-#if defined(CONF_ALLOWFILE)
-	allow_exists = FileExists(CONF_ALLOWFILE);
-#endif
-	in_deny = 0;
-	in_allow = 0;
-
-#if defined(CONF_DENYFILE)
-	if ( deny_exists )
-	{
-	        DEBUG_Str("Checking deny file for",user->pw_name);
-		in_deny = UserInFile(CONF_DENYFILE, user->pw_name);
-	}
-#endif
-#if defined(CONF_ALLOWFILE)
-	if ( allow_exists )
-	{
-	        DEBUG_Str("Checking allow file for",user->pw_name);
-		in_allow = UserInFile(CONF_ALLOWFILE, user->pw_name);
-	}
-#endif
-
-	if ( !deny_exists && !allow_exists )
-	{
-		Log(user->pw_name, "-", "access control files not found");
-#if defined(CONF_ALLOWFILE) && defined(CONF_DENYFILE)
-		MSG_Error_AccessControl("Access control files not found!",
-			CONF_ALLOWFILE " and " CONF_DENYFILE);
-#elif defined(CONF_ALLOWFILE)
-		MSG_Error_AccessControl("Access control file not found!",
-			CONF_ALLOWFILE);
-#elif defined(CONF_DENYFILE)
-		MSG_Error_AccessControl("Access control file not found!",
-			CONF_DENYFILE);
-#endif
-	}
-
-	if ( (in_allow && in_deny) ||
-		( allow_exists && !in_allow ) ||
-		( deny_exists && in_deny ) )
-	{
-		Log(user->pw_name, "-", "user/host not permitted");
-		MSG_Error_AccessControl("Script userid and/or remote host not permitted!",NULL);
-	}
-#endif
 
 #if defined(CONF_MINIMUM_UID)
+	DEBUG_Msg("Checking user minimum uid.");
 	if ( user->pw_uid < CONF_MINIMUM_UID )
 	{
 		Log(user->pw_name, "-", "uid less than minimum");
-		MSG_Error_AccessControl("UID of script userid less than configured minimum.",NULL);
+		MSG_Error_AccessControl("UID of script userid less than configured minimum.",
+			NULL, NULL);
 	}
 #endif
 
 #if defined(CONF_MINIMUM_GID)
+	DEBUG_Msg("Checking user minimum gid.");
 	if ( user->pw_gid < CONF_MINIMUM_GID )
 	{
 		Log(user->pw_name, "-", "gid less than minimum");
-		MSG_Error_AccessControl("GID of script userid less than configured minimum.",NULL);
+		MSG_Error_AccessControl("GID of script userid less than configured minimum.",
+			NULL, NULL);
 	}
 #endif
 
@@ -380,6 +389,7 @@ void CheckUser(struct passwd *user)
 	char *sh, *getusershell();
 	int found = 0;
 
+	DEBUG_Msg("Checking user shell.");
 	while ( sh = getusershell() )
 	{
 		if (0 == strcmp( sh, user->pw_shell ))
@@ -391,14 +401,191 @@ void CheckUser(struct passwd *user)
 	if (found == 0)
 	{
 	  Log(user->pw_name, "-", "restricted login shell");
-	  MSG_Error_AccessControl("Restricted login shell, permission denied.",NULL);
+	  MSG_Error_AccessControl("Restricted login shell, permission denied.",
+	  	NULL, NULL);
 	}
 	}
 #endif
 
+#if defined(CONF_REQUIRE_REDIRECT_URL)
+	DEBUG_Msg("Checking for required REDIRECT_URL.");
+	if ( ( !getenv("REDIRECT_URL") ) || (getenv("REDIRECT_URL")[0] == '\0') )
+	{
+		Log(user->pw_name, "-", "no redirect url");
+		MSG_Error_AccessControl("REDIRECT_URL not in environment, cgiwrap not invoked via Action/Handler.",
+			NULL, NULL);
+	}
+#endif
+
+#if defined(CONF_ENV_REQUIRE_USER)
+	DEBUG_Msg("Checking for CGIWRAP_REQUIRE_USER in environment.");
+	if ( getenv("CGIWRAP_REQUIRE_USER") )
+	{
+		if ( strcmp(getenv("CGIWRAP_REQUIRE_USER"), user->pw_name) )
+		{
+			Log(user->pw_name, "-", "CGIWRAP_REQUIRE_USER mismatch");
+			MSG_Error_AccessControl("User does not match CGIWRAP_REQUIRE_USER in environment.",
+				NULL, NULL);
+		}
+	}
+#endif
+
 }
 
 
+/*
+ * Perform checks on the userid (global checks)
+ */
+void CheckUserAccess(struct passwd *user)
+{
+#if defined(CONF_DENYFILE) || defined(CONF_ALLOWFILE)
+	char *denyfile = NULL;
+	char *allowfile = NULL;
+	
+#if defined(CONF_DENYFILE)
+	denyfile = CONF_DENYFILE;
+	DEBUG_Str("Global Deny File: ", denyfile);
+#endif
+#if defined(CONF_ALLOWFILE)
+	allowfile = CONF_ALLOWFILE;
+	DEBUG_Str("Global Allow File: ", allowfile);
+#endif
+
+	CheckAccess_Helper(user, allowfile, denyfile);
+#endif
+}
+
+/*
+ * Build paths of the vhost access control files, and check to
+ * see if they allow this access. If not found, fail. 
+ */
+void CheckVHostUserAccess(struct passwd *user)
+{
+#if defined(CONF_VHOST_ALLOWDIR) || defined(CONF_VHOST_DENYDIR)
+	char *denyfile = NULL;
+	char *allowfile = NULL;
+	char *http_host = getenv("HTTP_HOST");
+	char *lower_http_host = NULL;
+	int i;
+
+#ifdef CONF_VHOST_OVERRIDE
+	char *cgiwrap_auth_vhost = getenv("CGIWRAP_AUTH_VHOST");
+
+	if ( cgiwrap_auth_vhost && cgiwrap_auth_vhost[0])
+	{
+		DEBUG_Str("Overriding Access Control Virtual Host: ", HTMLEncode(cgiwrap_auth_vhost));
+		http_host = cgiwrap_auth_vhost;
+	}
+#endif
+
+	/* Get the vhost we are running on */
+	if ( ! http_host || ! http_host[0] )
+	{
+		MSG_Error_AccessControl(
+			"Cannot determine virtual host for access control.", NULL, NULL);
+	}
+	
+	/* Force to all lowercase */
+	DEBUG_Msg("Determining virtual host for access control.");
+	lower_http_host = (char *) SafeMalloc(strlen(http_host) + 1,
+		"lower_http_host");
+	for (i=0; i<=strlen(http_host); i++)
+	{
+		lower_http_host[i] = tolower(http_host[i]);
+	}
+
+	DEBUG_Msg("\n");
+	DEBUG_Str("Access Control Virtual Host: ", lower_http_host);
+
+	/* Now build the two filenames */	
+#if defined(CONF_VHOST_ALLOWDIR)
+	allowfile = (char *) SafeMalloc(strlen(CONF_VHOST_ALLOWDIR) + 1 +
+		strlen(lower_http_host) + 1, "vhost allowfile");
+	strcpy(allowfile, CONF_VHOST_ALLOWDIR);
+	strcat(allowfile, "/");
+	strcat(allowfile, lower_http_host);
+
+	DEBUG_Str("     VHost Allow File: ", allowfile);
+#endif
+
+#if defined(CONF_VHOST_DENYDIR)
+	denyfile = (char *) SafeMalloc(strlen(CONF_VHOST_DENYDIR) + 1 +
+		strlen(lower_http_host) + 1, "vhost denyfile");
+	strcpy(denyfile, CONF_VHOST_DENYDIR);
+	strcat(denyfile, "/");
+	strcat(denyfile, lower_http_host);
+
+	DEBUG_Str("     VHost Deny File: ", denyfile);
+#endif
+
+	CheckAccess_Helper(user, allowfile, denyfile);
+	
+	/* Clean up */
+	if ( denyfile ) free(denyfile);
+	if ( allowfile ) free(allowfile);
+	free(lower_http_host);
+#endif
+}
+
+void CheckAccess_Helper(struct passwd *user, char *allowfile, char *denyfile)
+{
+	int deny_exists=0, allow_exists=0;
+	int in_deny=0, in_allow=0;	
+
+	/* If neither specified, we're not configured for it */
+	if ( !allowfile && !denyfile )
+	{
+		return;
+	}
+
+	DEBUG_Msg("\nChecking Access Files:");
+
+	if ( denyfile )
+	{
+		deny_exists = FileExists(denyfile);
+		if ( deny_exists )
+		{
+			DEBUG_Str("  Deny file exists: ", denyfile);
+		}
+	}
+	if ( allowfile )
+	{
+		allow_exists = FileExists(allowfile);
+		if ( allow_exists )
+		{
+			DEBUG_Str("  Allow file exists: ", allowfile);
+		}
+	}
+
+	if ( !deny_exists && !allow_exists )
+	{
+		Log(user->pw_name, "-", "access control files not found");
+		MSG_Error_AccessControl("Access control files not found!",
+			allowfile, denyfile);
+	}
+
+	if ( deny_exists )
+	{
+	        DEBUG_Str("  Checking deny file for",user->pw_name);
+		in_deny = UserInFile(denyfile, user->pw_name);
+	}
+	if ( allow_exists )
+	{
+	        DEBUG_Str("  Checking allow file for",user->pw_name);
+		in_allow = UserInFile(allowfile, user->pw_name);
+	}
+
+	if ( (in_allow && in_deny) ||
+		( allow_exists && !in_allow ) ||
+		( deny_exists && in_deny ) )
+	{
+		Log(user->pw_name, "-", "user/host not permitted");
+		MSG_Error_AccessControl(
+			"Script userid and/or remote host not permitted!",
+			NULL, NULL);
+	}
+}
+
 
 /*
  * Perform file checks on the script file
@@ -451,7 +638,11 @@ void CheckScriptFile(void)
 			"Script is not a regular file");
 	}
 
-	if (!(fileStat.st_mode & S_IXUSR))
+	if (fileStat.st_mode & S_IXUSR)
+	{
+		Context.script_is_executable = 1; /* record this in case we might use interpreter later */
+	}
+	else if ( !Context.interpreted_script ) /* if we're passing to interpreter, isn't an error to be non-executable */
 	{
 		MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
 			"Script is not executable. Issue 'chmod 755 filename'");
@@ -650,7 +841,10 @@ char *StripPathComponents(int count, char *path)
 		{
 			found++;
 		}	
-		i++;
+		if ( found < count )
+		{
+			i++;
+		}
 	}
 	tmp = strdup(path + i);
 
@@ -865,11 +1059,12 @@ void ChangeAuxGroups(struct passwd *user)
 
 		for ( j=0; j<i; j++ )
 		{
-printf("checking %d against %d\n", groups[j], CONF_MINIMUM_GID);
 			if ( groups[j] < CONF_MINIMUM_GID )
 			{
                 		Log(user->pw_name, "-", "supplementary gid less than minimum");
-                		MSG_Error_AccessControl("Supplementary GID of script userid less than configured minimum.",NULL);
+                		MSG_Error_AccessControl(
+					"Supplementary GID of script userid less than configured minimum.",
+					NULL, NULL);
 			}
 		}
 	}
@@ -883,10 +1078,10 @@ printf("checking %d against %d\n", groups[j], CONF_MINIMUM_GID);
 int UserInFile(char *filename, char *user)
 {
 	FILE *file;
-	char temp[HUGE_STRING_LEN], *tmpuser;
+	char temp[HUGE_STRING_LEN];
 #if defined(CONF_CHECKHOST)
 	int count, remote_addr[4],spec_mask[4],spec_addr[4];
-	char *i;
+	char *i, *tmpuser;
 #endif
 	int j, intail;
 
@@ -939,9 +1134,23 @@ int UserInFile(char *filename, char *user)
 		if ( !strcmp(temp,user) )
 		{
 			fclose(file);
-			DEBUG_Str("Found",user);
+			DEBUG_Str("    Found",user);
 			return 1;
 		}
+
+#if defined(CONF_ALLOWDENY_NETGROUPS) && defined(HAS_INNETGR)
+		if ( strlen(temp) > 3 && temp[0] == '+' && temp[1] == '@' )
+		{
+			if ( innetgr(&temp[2], NULL, user, NULL) )
+			{
+				DEBUG_Str("    Found in netgroup ", user);
+				return 1;
+			}
+		}
+#elif defined(CONF_ALLOWDENY_NETGROUPS)
+#error netgroup support requested, but innetgr not available
+#endif
+
 #if defined(CONF_CHECKHOST)
 		i = strchr(temp, '@');
 		tmpuser = strtok(temp, "@");
@@ -1088,6 +1297,55 @@ void SetScriptName(char *userStr, char *scrStr )
 	char *buf;
 	char *name;
 
+#if defined(CONF_USE_REDIRECT_URL)
+	char *redurl = getenv("REDIRECT_URL");
+	int len;
+
+	/* Use REDIRECT_URL to build SCRIPT_NAME so we can
+	 * keep the URL the way user specified them.
+	 * For this to work we need to have already set
+	 * the correct PATH_INFO env if needed (this is
+ 	 * done by FetchScriptString).
+	 * NOTE: REDIRECT_URL is not a CGI standard
+	 * environment variable. --san@cobalt.rmnet.it;
+	 */
+	if ( redurl )
+	{
+		name = getenv("PATH_INFO");
+		if ( name ) {
+			/* We need to strip PATH_INFO from REDIRECT_URL */
+			len = strlen(redurl) - strlen(name);
+			buf = (char*) SafeMalloc (strlen("SCRIPT_NAME=") +
+		    		len + 2,
+				"new SCRIPT_NAME environment variable");
+			snprintf(buf, strlen("SCRIPT_NAME=")+len+1,
+				"SCRIPT_NAME=%s", redurl);
+			
+		}
+		else {
+			buf = (char*) SafeMalloc (strlen("SCRIPT_NAME=") +
+		    		strlen(redurl) + 2,
+				"new SCRIPT_NAME environment variable");
+			sprintf(buf, "SCRIPT_NAME=%s", redurl);
+		}
+		SafePutenv(buf, "set SCRIPT_NAME environment variable");
+		return;
+	}
+#endif
+
+#if defined(CONF_USE_SCRIPT_URL)
+	name = getenv("SCRIPT_URL");
+	if ( name ) {
+		buf = (char*) SafeMalloc (strlen("SCRIPT_NAME=") + strlen(name) + 3, 
+			"new SCRIPT_NAME environment variable");
+
+		sprintf(buf, "SCRIPT_NAME=%s", name);
+		SafePutenv(buf,"set SCRIPT_NAME environment variable");
+		return;
+	}
+
+#endif
+
 	name = getenv("SCRIPT_NAME");
 	if ( name ) {
 		/* only set SCRIPT_NAME if it was already set */
@@ -1130,23 +1388,61 @@ void SetScriptFilename (char *scriptPath)
  */
 void SetPathTranslated( char *cgiBaseDir, char *scriptPath )
 {
-	char *buf, *pathinfo;
-	
-	pathinfo = getenv("PATH_INFO");
+	char *buf;
+	char *old_pt, *new_pt;
+	char *old_pi, *new_pi;
+	char *docroot;
 
-	if ( !pathinfo )
+	old_pt = getenv("PATH_TRANSLATED");
+	if ( ! old_pt )
 	{
-		SafePutenv("PATH_TRANSLATED=", 
-			"set PATH_TRANSLATED environment variable");
+		/* don't set one if the server didn't */
+		return;
+	}
+	new_pt = strdup(old_pt);
+
+	old_pi = Context.origPathInfo;
+	new_pi = Context.newPathInfo;
+	if ( !old_pi || !new_pi )
+	{
+		/* can't match against anything */
 		return;
 	}
 
-	buf = (char *) SafeMalloc( strlen("PATH_TRANSLATED") +
-		strlen(cgiBaseDir) + strlen(pathinfo) + 5, 
-		"new PATH_TRANSLATED environment variable");
+	/* check if we find old path_info (with user) in the path_translated string */
+	buf = strstr(new_pt, old_pi);
+	if ( buf )
+	{
+		/* if so, copy in what we determined pathinfo should be after stripping off user portion */
+		if ( Context.interpreted_script ) /* for PHP we do not strip script path from PATH_TRANSLATED */
+		{
+			strcpy(buf, "/");
+			strcat(buf, Context.scriptRelativePath);
+			strcat(buf, new_pi);
+		} else {
+			strcpy(buf, new_pi);
+		}
 
-	sprintf(buf, "%s=%s%s", "PATH_TRANSLATED", cgiBaseDir, pathinfo); 
-	putenv(buf);
+		buf = (char *) SafeMalloc( strlen(new_pt) + strlen("PATH_TRANSLATED") + 5, 
+			"new PATH_TRANSLATED environment variable");
+		sprintf(buf, "%s=%s", "PATH_TRANSLATED", new_pt); 
+		SafePutenv(buf, "new PATH_TRANSLATED environment variable");
+
+		return;
+	}
+
+	/* we might be able to fall back to using docroot if we have it */
+
+	docroot = getenv("DOCUMENT_ROOT");
+	if ( docroot )
+	{
+		buf = (char *) SafeMalloc( strlen("PATH_TRANSLATED") + strlen(docroot) + 
+			strlen(new_pi) + 5, "new PATH_TRANSLATED environment variable");
+		sprintf(buf, "PATH_TRANSLATED=%s%s", docroot, new_pi);
+		SafePutenv(buf, "new PATH_TRANSLATED environment variable");
+
+		return;
+	}
 }
 
 
